<?php
/**
 * Creates and calls a java tag with the given attributes
 * You hav two ways to call a tag depending, if it supports
 * the body evalution (<ns:tag>...</ns:tag>) or not (<ns:tag/>)
 *
 * With BodyTagSupport (<core:form>...</core:form>):
 * $form = new Java_Tag($pageContext, "c4u.core.view.tags.form.FormTag");
 * if($form->start()) {
 * do {
 * } while($form->repeat());
 * $form->end();
 *
 * Without: (<core:image id="c4u.core.common.edit"/>)
 * $prop = array("id" => "c4u.core.common.edit" );
 * $imagetag = new Java_Tag($pageContext, "c4u.core.view.tags.ImageTag", $prop);
 * $imagetag->call();
 *
 * @author Alexander Merz <kontakt@alexander-merz.com>
 */
class Java_Tag {
    protected $jClass = null;
    protected $pc = null;
    protected $tagext = null;
    protected $tagbody = null;
    protected $noend = false;
    protected $evalTag = null;

    /**
     * Creates a wrapper for a JSP tag
     *
     * @param Object Java_Tomcat_PageContext $pageContext the java page context object
     * @param String $classname the classname of the tag
     * @param Array $attribute a hash containg the attributes as key => value pairs
     */
    function __construct($pageContext, $classname, $attribute = null) {
        $this->pc = $pageContext;
        $this->tagext = new JavaClass("javax.servlet.jsp.tagext.Tag");
        $this->tagbody = new JavaClass("javax.servlet.jsp.tagext.BodyTag");
        $this->jClass = new Java($classname);
        $this->jClass->setPageContext($pageContext->getPageContext());
        if(null != $attribute) {
            foreach($attribute as $k => $v) {
                $method = "set";
                $s = strtoupper(substr($k, 0, 1));
                $method .= $s.substr($k, 1);
                $this->jClass->$method($v);
            }
        }
    }

    /**
     * Calls a tag without BodyTagSupport
     * equal to <ns:tag /> in JSP
     *
     * @param bool $flush if true, the output of the tag is given to the JSPWriter immediatly
     */
    function call($flush = true) {
        $this->jClass->doStartTag();
        $this->jClass->doEndTag();
        if($flush) {
            $this->pc->getOut()->flush();
        }
    }

    /**
     * Calls the start tag
     * equal to <ns:tag> in JSP
     *
     * @return boolean if true, the tag body should be evaluated
     */
    function start() {

        $this->evalTag = $this->jClass->doStartTag();
        if($this->evalTag != $this->tagext->SKIP_BODY) {
            $this->noend=true;
            if($this->evalTag != $this->tagext->EVAL_BODY_INCLUDE) {
                $out = $this->pc->getPageContext->pushBody();
                $this->jClass->setBodyContent(out);
                $this->jClass->doInitBody();
            }
            return true;
        }
        return false;
    }

    /**
     * Calls the end tag
     * equal to </ns:tag> in JSP
     *
     * @return boolean if true, the tag body should be evalutes again
     */
    function repeat() {

            $evalDoAfterBody = $this->jClass->doAfterBody();
            if ($evalDoAfterBody != $this->tagbody->EVAL_BODY_AGAIN) {
                return false;
            }

        return true;
    }

    /**
    * Must be called after the last repeat()
    */
    function end($flush = true) {
        if(!$this->noend) {
                if ($this->evalTag != $this->tagext->EVAL_BODY_INCLUDE) {
                    $this->pc->getPageContext()->popBody();
                }
        }
        $this->jClass->doEndTag();
        if($flush ) {
            $this->pc->getOut()->flush();
        }
        return true;
    }


    // TODO: implement __call instead for calling methods directly per reflection
    /**
     * returns the instance of the tag class
     *
     * @return Object Java object the tag class instance
     */
    function getClass() {
        return $this->jClass;
    }

}
?>
