<?php
/**
 * Allows to buffer the output of the tag.
 * It buffers the output of the JSPWriter.
 * This class requires the com.alexandermerz.OutputBuffer java class!
 *
 * $tagbuffer = new Java_Tag_Buffer($pageContext);
 * $tagbuffer->start();
 *
 *   $prop = array("id" => "c4u.core.common.edit" );
 *   $imagetag = new Java_Tag($pageContext, "c4u.core.view.tags.ImageTag", $prop);
 *   $imagetag->call();
 *
 *  $tagbuffer->close();
 *  echo $out = $tagbuffer->getString();
 */
class Java_Tag_Buffer {

    protected $pc = null;
    protected $output = null;

    /**
     * Creates the buffer
     *
     * @param Object Java_Tomcat_PageContext $pc the PageContext object
     */
    function __construct($pc) {
        $this->pc = $pc;
        $this->output = new Java("com.alexandermerz.OutputBufferTag");
        $this->output->setPageContext($pc->getPageContext());
        $this->output->doStartTag();
        $this->output->setBodyContent($pc->getPageContext()->pushBody());
    }

    /**
     * Starts the buffering
     */
    function start() {
        $this->output->doStartTag();
        $this->output->setBodyContent($pc->getPageContext()->pushBody());
    }

    /**
     * Closes the buffer
     */
    function close() {
         $this->output->doInitBody();
         $this->output->doAfterBody();
         $this->output->doEndTag();
         $this->pc->getPageContext()->popBody();
    }

    /**
     * Returns the buffered content
     *
     * @return String returns the buffer content
     */
    function getString() {
        return $this->output->getString();
    }
}

?>
